 /**************************************************************************
 *                                                                         *
 *                                 RasTop                                  *
 *                                                                         *
 *                 Molecular Graphics Visualisation Tool                   *
 *                                                                         *
 *                           by Philippe Valadon                           *
 *                 P.O. Box 8064, La Jolla, CA 92192-8064, USA,            *
 *                       www.geneinfinity.org/rastop                       *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 *                   Based on RasMol 2.6 by Roger Sayle                    *
 * Biomolecular Structures Group, Glaxo Wellcome Research & Development,   *
 *                      Stevenage, Hertfordshire, UK                       *
 *         Version 2.6, August 1995, Version 2.6.4, December 1998          *
 *                   Copyright (C) Roger Sayle 1992-1999                   *
 *                                                                         *
 *                                 and                                     *
 *                                                                         *
 * Arne Mueller       RasMol 2.6x1   May 1998   (C) Arne Mueller 1998      *
 * Gary Grossman and  RasMol 2.5-ucb Nov 1995   (C) UC Regents/ModularCHEM *
 * Marco Molinaro     RasMol 2.6-ucb Nov 1996       Consortium 1995, 1996  *
 * Herbert J.         RasMol 2.7.0   Mar 1999   (C) Herbert J. Bernstein   * 
 * Bernstein          RasMol 2.7.1   Jun 1999       1998-2001              *
 *                    RasMol 2.7.1.1 Jan 2001                              *
 *                    RasMol 2.7.2   Aug 2000                              *
 *                    RasMol 2.7.2.1 Apr 2001                              *
 * Philippe Valadon   RasTop 2.0     Jan 2002   (C) Philippe Valadon       *
 *                                                  2000-2002              *
 *                                                                         *
 * Naoum Salam       RasTop 2.0.1-VF May 2002  (C) Naoum Salam 2002      *
 *                                                                         *
 * Please read the file NOTICE for important notices which apply to this   *
 * package. If you are not going to make changes to Rasmol, you are not    *
 * only permitted to freely make copies and distribute them, you are       *
 * encouraged to do so, provided you do the following:                     *
 *   * 1. Either include the complete documentation, especially the file   *
 *     NOTICE, with what you distribute or provide a clear indication      *
 *     where people can get a copy of the documentation; and               *
 *   * 2. Please give credit where credit is due citing the version and    *
 *     original authors properly; and                                      *
 *   * 3. Please do not give anyone the impression that the original       *
 *     authors are providing a warranty of any kind.                       *
 *                                                                         *
 * If you would like to use major pieces of Rasmol in some other program,  *
 * make modifications to Rasmol, or in some other way make what a lawyer   *
 * would call a "derived work", you are not only permitted to do so, you   *
 * are encouraged to do so. In addition to the things we discussed above,  *
 * please do the following:                                                *
 *   * 4. Please explain in your documentation how what you did differs    *
 *     from this version of Rasmol; and                                    *
 *   * 5. Please make your modified source code available.                 *
 *                                                                         *
 * This version of Rasmol is not in the public domain, but it is given     *
 * freely to the community in the hopes of advancing science. If you make  *
 * changes, please make them in a responsible manner, and please offer us  *
 * the opportunity to include those changes in future versions of Rasmol.  *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// rasdoc.h
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_RASDOC_H_INCLUDED_)
#define AFX_RASDOC_H_INCLUDED_

#include <afxtempl.h>
#include "datadisplay.h"

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

char *RegisterFormat( char *buffer, char *desc, char *ext );
BOOL LoadInputFile(int format);

class CRasDoc : public CDocument
{
protected:
	CRasDoc();
	DECLARE_DYNCREATE(CRasDoc)

// Attributes
public:

// Operations
public:


// Overrides
	//{{AFX_VIRTUAL(CRasDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void OnCloseDocument();
	protected:
	virtual BOOL SaveModified();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CRasDoc();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

public:
	int m_DocCopyNum;
	CData m_DataDisplay;
	int ViewActivationFlag;
	HBITMAP m_PixMap;

	static CRasDoc* OpenRSMFile(CString& PathName);
	void FixRange();
	void ManageDisplay(BOOL ActivationFlag);
    int GetMaxCBFactor();

protected:
	static CRasDoc* GetEmptyDoc();
	void CheckActivationState();
	int CountDuplicates();
	int ManageOpening(CString& PathName);
	int GetSaveAsFilename( CString &Filename, CString &NewFilename );

protected:
	//{{AFX_MSG(CRasDoc)
	afx_msg void OnSaveMolecularfile();
	afx_msg void OnUpdateSaveMolecularfile(CCmdUI* pCmdUI);
	afx_msg void OnSaveImage();
	afx_msg void OnUpdateSaveImage(CCmdUI* pCmdUI);
	afx_msg void OnSaveScript();
	afx_msg void OnUpdateSaveScript(CCmdUI* pCmdUI);
	afx_msg void OnFileLoadRasmolscript();
	afx_msg void OnLoadMolecularfile();
	afx_msg void OnFileOpenFromDoc();
	afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileSaveAs(CCmdUI* pCmdUI);
	afx_msg void OnFileSave();
	afx_msg void OnFileSaveAs();
	afx_msg void OnFileMoleculeAs();
	afx_msg void OnUpdateFileMoleculeAs(CCmdUI* pCmdUI);
	afx_msg void OnFileAdd();
	afx_msg void OnUpdateFileAdd(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}

#endif // !defined(AFX_RASDOC_H_INCLUDED_)
